#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#define MAXN 100000
#define MAXM 500000

#define INP "asfalt.in"
#define OUT "asfalt.out"

/* Mezi kterymi vrcholy hrana vede a s kterou je sparovana (-1 pokud dosud neni) */
struct edge {
	int a;
	int b;
	int pair;
};

/* Pocatek hran od daneho vrcholu v seznamu hran a stupen vrcholu */
struct vertex {
	int start;
	int deg;
	int visited;
};

struct edge et[MAXM];
int ep[2*MAXM];
struct vertex v[MAXN];
int vn, en;

/* Nacti vstup a vytvor reprezentaci grafu */
void read_input(void)
{
	FILE *inp;
	int i;

	if (!(inp = fopen(INP, "r"))) {
		perror("Cannot open input file");
		exit(1);
	}
	fscanf(inp, "%d %d", &vn, &en);
	/* Otestujeme, jestli je uloha zjevne neresitelna */
	if (en & 1) {
		FILE *out;

		if (!(out = fopen(OUT, "w"))) {
			perror("Cannot open output file");
			exit(1);
		}
		fputs("Cesty nelze vyasfaltovat.\n", out);
		exit(0);
	}
	for (i = 0; i < en; i++) {
		fscanf(inp, "%d %d", &et[i].a, &et[i].b);
		et[i].a--; et[i].b--;
		et[i].pair = -1;
		v[et[i].a].deg++;
		v[et[i].b].deg++;
	}
	fclose(inp);
	/* Seradime si pole s hranami */
	for (i = 1; i < vn; i++) {
		v[i].start = v[i-1].start+v[i-1].deg;
		v[i-1].deg = 0;
	}
	v[vn-1].deg = 0;
	for (i = 0; i < en; i++) {
		ep[v[et[i].a].start+v[et[i].a].deg++] = i;
		ep[v[et[i].b].start+v[et[i].b].deg++] = i;
	}
}

/* Vrati druhy konec hrany */
int edge_end(int v, int edgenum)
{
	if (et[edgenum].a != v)
		return et[edgenum].a;
	return et[edgenum].b;
}

/* Vytiskne vysledek */
void print_out(void)
{
	int i, tmp;
	FILE *out;

	if (!(out = fopen(OUT, "w"))) {
		perror("Cannot open output file");
		exit(1);
	}
	for (i = 0; i < en; i++)
		if (et[i].pair > i) {
			int v[4] = { et[i].a, et[i].b, et[et[i].pair].a, et[et[i].pair].b };

			if (v[0] == v[2] || v[0] == v[3]) {
				tmp = v[0];
				v[0] = v[1];
				v[1] = tmp;
			}
			if (v[0] == v[3] || v[1] == v[3]) {
				tmp = v[2];
				v[2] = v[3];
				v[3] = tmp;
			}
			fprintf(out, "%d %d %d\n", v[0]+1, v[1]+1, v[3]+1);
		}
	fclose(out);
}

/* Prohledavani do hloubky na parovani hran */
void dfs(int act, int parent)
{
	int i, pair = -1;

	v[act].visited = 1;
	/* Zavolame se na vsechny sousedni vrcholy, ve kterych jsme dosud nebyli */
	for (i = v[act].start; i < v[act].start+v[act].deg; i++)
		if (!v[edge_end(act, ep[i])].visited)
			dfs(edge_end(act, ep[i]), act);
	/* Nyni sparujeme zbyle hrany */
	for (i = v[act].start; i < v[act].start+v[act].deg; i++)
		if (et[ep[i]].pair == -1 && edge_end(act, ep[i]) != parent) {
			if (pair == -1)	/* Nemame hranu ke sparovani? */
				pair = ep[i];
			else {
				et[ep[i]].pair = pair;
				et[pair].pair = ep[i];
				pair = -1;
			}
		}
	if (pair != -1) {	/* Hrana zbyla? */
		/* Sparujeme ji s hranou k rodici */
		for (i = v[act].start; edge_end(act, ep[i]) != parent; i++);
		et[ep[i]].pair = pair;
		et[pair].pair = ep[i];
	}
}

int main(void)
{
	read_input();
	dfs(0, 0);
	print_out();
	return 0;
}
