#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>

#define MAX_LINE_LEN 100

#define MAXN 100000
#define MAXM 500000

FILE *answer;

/* Mezi kterymi vrcholy hrana vede a zda jiz byla sparovana */
struct edge {
	int a;
	int b;
	int used;
};

/* Pocatek hran od daneho vrcholu v seznamu hran a stupen vrcholu */
struct vertex {
	int start;
	int deg;
};

struct edge et[MAXM];
int ep[2*MAXM];
struct vertex v[MAXN];
int vn, en;

void die(char *fmtstr, ...)
{
	va_list args;

	va_start(args, fmtstr);
	vprintf(fmtstr, args);
	va_end(args);
	exit(1);
}

/* Nacti vstup a vytvor reprezentaci grafu */
int read_input(char *name)
{
	FILE *inp;
	int i;

	if (!(inp = fopen(name, "r"))) {
		perror("Cannot open input file");
		exit(1);
	}
	fscanf(inp, "%d %d", &vn, &en);
	/* Otestujeme, jestli je uloha zjevne neresitelna */
	if (en & 1)
		return 1;
	for (i = 0; i < en; i++) {
		fscanf(inp, "%d %d", &et[i].a, &et[i].b);
		et[i].a--; et[i].b--;
		v[et[i].a].deg++;
		v[et[i].b].deg++;
	}
	fclose(inp);
	/* Seradime si pole s hranami */
	for (i = 1; i < vn; i++) {
		v[i].start = v[i-1].start+v[i-1].deg;
		v[i-1].deg = 0;
	}
	v[vn-1].deg = 0;
	for (i = 0; i < en; i++) {
		ep[v[et[i].a].start+v[et[i].a].deg++] = i;
		ep[v[et[i].b].start+v[et[i].b].deg++] = i;
	}
	return 0;
}

/* Vrati druhy konec hrany */
int edge_end(int v, int edgenum)
{
	if (et[edgenum].a != v)
		return et[edgenum].a;
	return et[edgenum].b;
}

void test_edge(int line, int a, int b)
{
	int i;

	for (i = 0; i < v[a].deg && edge_end(a, ep[i+v[a].start]) != b; i++);
	if (i >= v[a].deg)
		die("radek %d: (%d,%d) neni hrana.\n", line+1, a+1, b+1);
	if (et[ep[i+v[a].start]].used)
		die("radek %d: hrana (%d,%d) jiz byla pouzita na radku %d.\n", line+1, a+1, b+1, et[ep[i+v[a].start]].used);
	et[ep[i+v[a].start]].used = line+1;
}

void test_input(void)
{
	int i, a, b, c;
	char buf[MAX_LINE_LEN];

	for (i = 0; i < en/2; i++) {
		if (!fgets(buf, sizeof(buf), answer))
			die("Prilis malo radku ve vystupu - bylo %d, ocekavano %d.\n", i, en/2);
		if (buf[strlen(buf)-1] != '\n')
			die("Radek %d je moc dlouhy.\n", i+1);
		if (sscanf(buf, "%d %d %d", &a, &b, &c) != 3)
			die("Nemohu precist cisla vrcholu z radku %d.\n", i+1);
		if (a < 1 || a > vn || b < 1 || b > vn || c < 1 || c > vn)
			die("Cisla vrcholu (%d,%d,%d) na radku %d jsou moc mala/velka (limit <1,%d>).\n", a,b,c,i+1,vn);
		a--; b--; c--;
		test_edge(i, a, b);
		test_edge(i, b, c);
	}
	if (fgets(buf, sizeof(buf), answer))
		die("Prilis mnoho radku ve vystupu - ocekavano %d.\n", en/2);
	printf("Ok\n");
}

int main(int argc, char **argv)
{
	char buf[MAX_LINE_LEN];

	answer = fopen(argv[2], "r");
	if (read_input(argv[1])) {
		char *c;
		fgets(buf, sizeof(buf), answer);
		if (c = strchr(buf, '\n'))
			*c = 0;
		if (strcmp("Cesty nelze vyasfaltovat.", buf))
			die("Reseni neexistuje, ale odpoved je jina.\n");
		printf("Ok\n");
		return 0;
	}
	test_input();
	return 0;
}
